<?php
/**
 * Payment storage implementation for PHP's PEAR database abstraction layer.
 *
 * Required values for $params:<pre>
 *      'phptype'       The database type (e.g. 'pgsql', 'mysql', etc.).
 *      'charset'       The database's internal charset.</pre>
 *
 * Required by some database implementations:<pre>
 *      'database'      The name of the database.
 *      'hostspec'      The hostname of the database server.
 *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.</pre>
 *
 * The table structure can be created by the scripts/drivers/payment_foo.sql
 * script.
 *
 * $Horde: payment/lib/Driver/sql.php,v 1.12 2006/03/07 18:17:34 slusarz Exp $
 *
 * @author  Duck <duck@obala.net>
 * @package Horde_Payment
 */
class Payment_Driver_sql extends Payment_Driver {

    /**
     * Hash containing connection parameters.
     *
     * @var array
     */
    private $params = array();

    /**
     * Handle for the current database connection.
     *
     * @var DB
     */
    private $db;

    /**
     * Handle for the current database connection, used for writing. Defaults
     * to the same handle as $db if a separate write database is not required.
     *
     * @var DB
     */
    private $writedb;

    /**
     * Constructs a new SQL storage object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function Payment_Driver_sql($params = array())
    {
        $this->params = $params;
        $this->connect();
    }

    /**
    * get Authorization data
    */
    function getAuthorization($id)
    {
        $sql = 'SELECT * FROM payment_authorizations WHERE id = ?';
        $result = $this->db->getRow($sql, array($id), DB_FETCHMODE_ASSOC);
        if (empty($result)) {
            return PEAR::raiseError(sprintf(_("No transaction \"%s\" found"), $id));
        }

        return $result;
    }

    /**
    * list Authorizations
    */
    function listAuthorizations($criteria = array())
    {
        $sql = 'SELECT * FROM payment_authorizations';

        foreach ($criteria as $key => $val) {
            if (empty($val)) {
                unset($criteria[$key]);
            }
        }

        if (!empty($criteria)) {
            $sql .= ' WHERE ' . implode(' = ? AND ', array_keys($criteria)) . ' = ? ';
        }

        return $this->db->getAll($sql, $criteria, DB_FETCHMODE_ASSOC);
    }

    /**
    * Save authorization data
    *
    * @param string $api     The Horde application that is doing the authorization
    * @param string $id      Horde appliaction internal id
    * @param float  $amount  Amount for atuherisation request
    */
    function addAuthorization($api, $id, $amount)
    {
        $sql = 'SELECT COUNT(*) FROM payment_authorizations WHERE module_name = ? AND module_id = ?';      
        $result = $this->db->getOne($sql, array($api, $id));
        if ($result == 1) {
            $name = $GLOBALS['registry']->get('name', $api);
            return PEAR::raiseError(sprintf(_("ID \"%s\" already exists in \"%s\""), $id, $name));
        }
        
        $sql = 'INSERT INTO payment_authorizations (id, module_name, module_id, amount, status, created) '
             . 'VALUES (?, ?, ?, ?, ?, ?)';

        $uid = uniqid();
        $values = array($uid, $api, $id, (int)$amount, PAYMENT_STATUS_PENDING, time());

        $result = $this->writedb->query($sql, $values);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        } else {
            return $uid;
        }
    }

    /**
    * Update authorization data
    *
    * @param string $id      Authorization ID
    * @param array  $data    Data tu update
    */
    function updateAuthorization($id, $data)
    {
        $values = array(time());
        $sql = 'UPDATE payment_authorizations SET updated = ? ';

        foreach ($data as $key => $value) {
            $sql .= ', ' . $key . ' = ? ';
            $values[] = $value;
        }

        $sql .=  'WHERE id = ?';
        $values[] = $id;
        
        return $this->writedb->query($sql, $values);
    }

    /**
    * Save authorization attributes data
    *
    * @param string $id      Authorization ID
    * @param array  $info    Infomation to store
    */
    function updateAuthorizationAttributes($id, $info)
    {
        $sql = 'INSERT INTO payment_authorizations_attributes (id, attribute_key, attribute_value) VALUES (?, ?, ?)'; 
        $sth = $this->writedb->prepare($sql);
        if (is_a($sth, 'PEAR_Error')) {
            return $sth;
        }

        $data = array();
        foreach ($info as $key => $value) {
            $data[] = array($id, $key, $value);
        }

        $sql = 'DELETE FROM payment_authorizations_attributes WHERE id = ?';
        $result = $this->writedb->query($sql, array($id));
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        return $this->writedb->executeMultiple($sth, $data);
    }


    /**
    * Get authorization attributes
    *
    * @param string $id      Authorization ID
    * @param array  $info    Infomation to store
    */
    function getAuthorizationAttributes($id)
    {
        $sql = 'SELECT attribute_key, attribute_value FROM payment_authorizations_attributes WHERE id = ?'; 
        return $this->db->getAssoc($sql, false, array($id));
    }


    /**
    * Save module method links
    *
    * @param array  $link    Matrix of payment methods
    */
    function saveMethodLink($link)
    {
        $sql = 'INSERT INTO payment_methods (module, method) VALUES (?, ?)'; 
        $sth = $this->writedb->prepare($sql);
        if (is_a($sth, 'PEAR_Error')) {
            return $sth;
        }

        $data = array();
        foreach ($link as $module => $methods) {
            foreach (array_keys($methods) as $method) {
                $data[] = array($module, $method);
            }        
        }

        $sql = 'TRUNCATE payment_methods';
        $result = $this->writedb->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
        
        return $this->writedb->executeMultiple($sth, $data);
    }

    /**
    * Get methods linked to app
    *
    * @param string  $app Aplication to ceck
    */
    function getMethods($app)
    {
        $sql = 'SELECT method FROM payment_methods WHERE module = ?';
        return $this->db->getCol($sql, 0, array($app));
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     * 
     * @return boolean  True on success; exits (Horde::fatal()) on error.
     */
    private function connect()
    {
        Horde::assertDriverConfig($this->params, 'storage',
                                  array('phptype', 'charset'));

        if (!isset($this->params['database'])) {
            $this->params['database'] = '';
        }
        if (!isset($this->params['username'])) {
            $this->params['username'] = '';
        }
        if (!isset($this->params['hostspec'])) {
            $this->params['hostspec'] = '';
        }

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->writedb = &DB::connect($this->params,
                                        array('persistent' => !empty($this->params['persistent'])));
        if (is_a($this->writedb, 'PEAR_Error')) {
            Horde::fatal($this->writedb, __FILE__, __LINE__);
        }

        // Set DB portability options.
        switch ($this->writedb->phptype) {
        case 'mssql':
            $this->writedb->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
            break;
        default:
            $this->writedb->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
        }

        /* Check if we need to set up the read DB connection seperately. */
        if (!empty($this->params['splitread'])) {
            $params = array_merge($this->params, $this->params['read']);
            $this->db = &DB::connect($params,
                                      array('persistent' => !empty($params['persistent'])));
            if (is_a($this->db, 'PEAR_Error')) {
                Horde::fatal($this->db, __FILE__, __LINE__);
            }

            // Set DB portability options.
            switch ($this->db->phptype) {
            case 'mssql':
                $this->db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
                break;
            default:
                $this->db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
            }

        } else {
            /* Default to the same DB handle for the writer too. */
            $this->db =& $this->writedb;
        }

        $this->connected = true;

        return true;
    }
}
